 /*
 *  file      : cd_cddb.c
 *  project   : xcfa
 *  with      : Gtk-2
 *
 *  copyright : (C) 2003 - 2011 by Claude Bulin
 *
 *  xcfa - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of XCFA.
 * 
 *  XCFA is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  XCFA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with XCFA.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <glib/gprintf.h>
#include <stdlib.h>
#include <stdio.h>
#include <linux/cdrom.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <cddb/cddb.h>


#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib/lib.h"
#endif

#include "global.h"
#include "cd_audio.h"
#include "cd_cddb.h"
#include "get_info.h"
#include "cd_cue.h"
#include "configuser.h"



#define CDDA_DATA_SIZE 2352





typedef struct {
	gchar		*Duree;
	gchar		*Artist;
	gchar		*Title;
	gint		Sector;
	gint		SectorCdParanoia;
} VAR_STRUCT_CDDB;

typedef struct {
	gint		TotalTracks;
	gchar		Message [ 100 ];
	gchar		*TitleCD;
	gchar		*Title;
	gchar		*Artiste;
	gchar		*StrDureeCd;
	gint		NumGenre;
	gchar		*StrGenre;
	gint		NumYear;
	gchar		*StrYear;
	gboolean	BoolMultiArtiste;
	
	VAR_STRUCT_CDDB	*VarStructCddb;		// 0 .. TotalTracks -1
	
} VAR_CDDB;

VAR_CDDB VarCddb = {
		0,
		"",
		NULL,
		NULL,
		NULL,
		NULL,
		0,
		NULL,
		0,
		NULL,
		FALSE
		};



// 
// 
void CdCddb_init_entete_cd (void)
{
	if( NULL != EnteteCD.TitleCD ) {
		g_free( EnteteCD.TitleCD );
		EnteteCD.TitleCD = NULL;
	}
	if( NULL != EnteteCD.Title ) {
		g_free( EnteteCD.Title );
		EnteteCD.Title = NULL;
	}
	if( NULL != EnteteCD.Artiste ) {
		g_free( EnteteCD.Artiste );
		EnteteCD.Artiste = NULL;
	}
	if( NULL != EnteteCD.StrDureeCd ) {
		g_free( EnteteCD.StrDureeCd );
		EnteteCD.StrDureeCd = NULL;
	}
	if( NULL != EnteteCD.StrGenre ) {
		g_free( EnteteCD.StrGenre );
		EnteteCD.StrGenre = NULL;
	}
	if( NULL != EnteteCD.StrYear ) {
		g_free( EnteteCD.StrYear );
		EnteteCD.StrYear = NULL;
	}
	
	cdaudio_deallocate_glist();
	
	if( NULL != EnteteCD.Message ) {
		g_free( EnteteCD.Message );
		EnteteCD.Message = NULL;
	}
	if( NULL != EnteteCD.StrMemoTagComment ) {
		g_free( EnteteCD.StrMemoTagComment );
		EnteteCD.StrMemoTagComment = NULL;
	}
}
// 
// 
void CdCddb_remove_struct( void )
{
	gint	NumList;
	
	VarCddb . TotalTracks = 0;
	VarCddb . Message [ 0 ] = '\0';
	if( NULL != VarCddb . TitleCD )		{ g_free( VarCddb . TitleCD );		VarCddb . TitleCD = NULL; }
	if( NULL != VarCddb . Title )		{ g_free( VarCddb . Title );		VarCddb . Title = NULL; }
	if( NULL != VarCddb . Artiste )		{ g_free( VarCddb . Artiste );		VarCddb . Artiste = NULL; }
	if( NULL != VarCddb . StrDureeCd )	{ g_free( VarCddb . StrDureeCd );		VarCddb . StrDureeCd = NULL; }
	if( NULL != VarCddb . StrGenre )	{ g_free( VarCddb . StrGenre );		VarCddb . StrGenre = NULL; }
	if( NULL != VarCddb . StrYear )		{ g_free( VarCddb . StrYear );		VarCddb . StrYear = NULL; }
	
	if( NULL != VarCddb . VarStructCddb ) {
		for( NumList = 0; NumList < VarCddb . TotalTracks; NumList++ ) {
			
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Duree ) {
				g_free( VarCddb . VarStructCddb [ NumList ] . Duree );		VarCddb . VarStructCddb [ NumList ] . Duree = NULL;
			}
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Artist ) {
				g_free( VarCddb . VarStructCddb [ NumList ] . Artist );		VarCddb . VarStructCddb [ NumList ] . Artist = NULL;
			}
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Title ) {
				g_free( VarCddb . VarStructCddb [ NumList ] . Title );		VarCddb . VarStructCddb [ NumList ] . Title = NULL;
			}
		}
		g_free( VarCddb . VarStructCddb );	VarCddb . VarStructCddb = NULL;
	}
}
// 
// 
gint cdparanoia_get_total_tracks( gchar *buffer )
{
	gchar  **Larrbuf = NULL;
	gint     Track;
	gint     TotalTracks = -1;
	gchar    *ptr;

	Larrbuf = g_strsplit( buffer, "\n", 0 );

	for( Track = 0; Larrbuf[ Track ]; Track++ ) {
		if( strstr (Larrbuf[ Track ], "TOTAL" )) {
			Track --;
			break;
		}
	}
	if( NULL != ( ptr = Larrbuf[ Track ] )) {
		while( *ptr == ' ' ) ptr ++;
		TotalTracks = atoi( ptr );
	}
	g_strfreev( Larrbuf );
	
	if( -1 != TotalTracks ) {
		// ALLOCATE DATAS SECTORS
		VarCddb . VarStructCddb = ( VAR_STRUCT_CDDB * ) g_malloc0( sizeof( VAR_STRUCT_CDDB ) * ( TotalTracks + 6 ));
	
		// INIT DATAS SECTORS
		for( Track = 0 ; Track < TotalTracks + 4; Track++ ) {
			VarCddb . VarStructCddb [ Track ] . Duree   = NULL;
			VarCddb . VarStructCddb [ Track ] . Artist  = NULL;
			VarCddb . VarStructCddb [ Track ] . Title   = NULL;
			VarCddb . VarStructCddb [ Track ] . Sector = -1;
			VarCddb . VarStructCddb [ Track ] . SectorCdParanoia = -1;
		}
	}
	
	return( TotalTracks );
}
// 
// 
void cdparanoia_get_sectors ( gchar *buffer )
{
	gchar	**Larrbuf = NULL;
	gint	piste;
	gchar	*ptr = NULL;
	gint	Track = 0;
	
	Larrbuf = g_strsplit( buffer, "\n", 0 );
	for( piste = 0; Larrbuf[ piste ]; piste ++ ) {
		if( strstr( Larrbuf[ piste ], "TOTAL" )) {
			ptr = Larrbuf[ piste ];
			while( *ptr && *ptr != ' ' ) ptr ++;
			while( *ptr && *ptr == ' ' ) ptr ++;
			if( *ptr ) {
				VarCddb . VarStructCddb [ Track ] . SectorCdParanoia = atoi( ptr );
			}
			break;
		}
		
		if( NULL != ( ptr = strchr( Larrbuf[ piste ], ']' ))) {
			ptr ++;
			while( *ptr && *ptr == ' ' ) ptr ++;
			if( *ptr )
				VarCddb . VarStructCddb [ Track ] . SectorCdParanoia = atoi( ptr );
				Track ++;
		}
	}
	g_strfreev( Larrbuf );
}

// RETURN
// 	0  OK
// 	1  PROBLEME D'ALLOCATION MEMOIRE
// 	2  CD NON AUDIO OU CD ABSENT DU LECTEUR
// 	3  LE CD DANS LECTEUR N'EST PAS UN CD-AUDIO
//	4  PAS DE CONNEXION WEB
//	5  PAS DE BASE POUR CE DISQUE
//	6  ERREUR DE LECTURE DEPUIS LE CD
//	7  CD ABSENT DU LECTEUR
// 
gint CdCddb_read_datas (void)
{
	cddb_conn_t	*p_cddb = NULL;
   	cddb_disc_t	*p_disc = NULL;
   	gint		Track;
	gint		drive;
	gint		NbrMatchsFound;
	gint		min;
	gint		sec;
	gint		som;
	struct	cdrom_tochdr	tochdr;
	struct	cdrom_tocentry	tocentry;
	struct	cdrom_subchnl	cdsc;
	gint		len;
	gint		TempDiv;
	gint		TempMod;
	gint		RetValue = 0;
	GString		*gstr = NULL;
	gint		TotalTracks;
	
	PRINT_FUNC_LF();
	
	// REMOVE VarCddb
	CdCddb_remove_struct();
	
	// GET REAL TRACKS AND GET REAL SECTORS FROM CDPARANOIA
	gstr = GetInfo_cdparanoia (EnteteCD.NameCD_Device);
	g_print("gstr->str=\n%s\n", gstr->str);
	if( ( TotalTracks = cdparanoia_get_total_tracks( gstr->str ) ) > -1 ) {
		cdparanoia_get_sectors( gstr->str );
	}
	g_string_free (gstr, TRUE);
	gstr = NULL;
	
	if( ( drive = open (EnteteCD.NameCD_Device, O_RDONLY | O_NONBLOCK) ) < 0 ) {
		g_print( "CD ABSENT DU LECTEUR\n" );
		strcpy( VarCddb . Message, _("CD absent du lecteur") );
		RetValue = 7;
		return( RetValue );
	}
	
	// ? CD NON AUDIO OU CD ABSENT DU LECTEUR
	cdsc.cdsc_format = CDROM_LBA;
	if( ioctl( drive, CDROMSUBCHNL, &cdsc ) < 0 ) {
 		g_print( "CD NON AUDIO OU CD ABSENT DU LECTEUR\n" );
 		strcpy( VarCddb . Message, _("CD non audio ou CD absent du lecteur") );
   		close( drive );
   		RetValue = 2;
		return( RetValue );
	}
	
	// GET TotalTracks
	ioctl( drive, CDROMREADTOCHDR, &tochdr );
	VarCddb . TotalTracks = tochdr.cdth_trk1;
	g_print( "VarCddb . TotalTracks       = %d\n",  VarCddb . TotalTracks );
	g_print( "TotalTracks with cdparanoia = %d\n",  TotalTracks );
	
	// ? LE CD DANS LECTEUR N'EST PAS UN CD-AUDIO
	tocentry.cdte_track = 0;
	tocentry.cdte_format = CDROM_MSF;
	ioctl(drive, CDROMREADTOCENTRY, &tocentry);
	min  = tocentry.cdte_addr.msf.minute;
	sec  = tocentry.cdte_addr.msf.second;
	som  = tocentry.cdte_addr.msf.frame;
	som += min * 60 * 75;
	som += sec * 75;
	if( 1 == VarCddb . TotalTracks && 150 == som ) {
		g_print( "LE CD DANS LECTEUR N'EST PAS UN CD-AUDIO\n" );
 		strcpy( VarCddb . Message, _("Le CD dans le lecteur n'est pas un CD-AUDIO") );
		close( drive );
		RetValue = 3;
		return( RetValue );
	}
	
	// ACDQUISITION DES SECTEURS
	for( Track = 0 ; Track <= VarCddb . TotalTracks ; Track++ ) {
		tocentry.cdte_format = CDROM_LBA;
		tocentry.cdte_track = ( Track == VarCddb . TotalTracks ) ? CDROM_LEADOUT : tochdr.cdth_trk0 + Track;
		if( ioctl( drive, CDROMREADTOCENTRY, &tocentry ) == -1 ) {
			g_print( "could not read TOCENTRY\n" );
			g_print( "ERREUR DE LECTURE DEPUIS LE CD\n" );
 			strcpy( VarCddb . Message, _("Erreur de lecture depuis le CD") );
   			close( drive);
   			RetValue = 6;
			return( RetValue );
		}
		
		VarCddb . VarStructCddb [ Track ] . Sector = tocentry.cdte_addr.lba;
		// g_print( "sectors [ %02d ] = %d\n", Track, VarCddb . VarStructCddb [ Track ] . Sector );
	}
	
	close( drive );
	
	// CALL LIBCDDB
	if( NULL != ( p_cddb = cddb_new() ) ) {
		if( SERVER_CDDB_PARAM == Config.ServeurCddb ) {
			cddb_set_server_name( p_cddb, Config.entry_cddp_server );
			g_print("With SERVER_CDDB_PARAM      = %s\n", Config.entry_cddp_server);
		}
		else {
			cddb_set_server_name( p_cddb, "freedb.org" );
		}
		cddb_set_server_port( p_cddb, 888 );
		cddb_cache_disable( p_cddb );
		cddb_set_timeout( p_cddb, 10 );
		cddb_http_disable( p_cddb );
	
		if( NULL != ( p_disc = cddb_disc_new() ) ) {
			int64_t i_length = 0;
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				cddb_track_t *t = cddb_track_new();
				cddb_track_set_frame_offset( t, VarCddb . VarStructCddb [ Track ] . Sector );
				cddb_disc_add_track( p_disc, t );
				const int64_t i_size = ( VarCddb . VarStructCddb [ Track +1 ] . Sector - VarCddb . VarStructCddb [ Track ] . Sector ) *(int64_t)CDDA_DATA_SIZE;
				i_length += 1000000 * i_size / 44100 / 4;
			}

			cddb_disc_set_length( p_disc, (int)( i_length / 1000000 ) );
			cddb_disc_calc_discid( p_disc );
			
			g_print( "-------------------------------------------------\n" );
			NbrMatchsFound = cddb_query( p_cddb, p_disc );
			g_print( "cddb_query()   = %d\n", NbrMatchsFound );
			g_print( "NbrMatchsFound = %d\n", NbrMatchsFound );
			if( NbrMatchsFound < 0 ) {
				g_print( "LA LIAISON INTERNET EST ABSENTE\n" );
 				strcpy( VarCddb . Message, _("La liaison internet est absente") );
				// cddb_disc_destroy( p_disc );
				// cddb_destroy( p_cddb );
				RetValue = 4;
				// return( RetValue );
			}
			
			if( NbrMatchsFound < 0 ) {
				g_print( "LA LIAISON INTERNET EST ABSENTE\n" );
 				strcpy( VarCddb . Message, _("La liaison internet est absente") );
				// cddb_disc_destroy ( p_disc );
				// cddb_destroy( p_cddb );
				RetValue = 4;
				// return( RetValue );
			}
			else if( NbrMatchsFound == 0 ) {
				g_print( "PAS DE BASE CDDB POUR CE DISQUE\n" );
 				strcpy( VarCddb . Message, _("Pas de base CDDB pour ce disque") );
				// cddb_disc_destroy( p_disc );
				// cddb_destroy( p_cddb );
				RetValue = 5;
				// return( RetValue );
			}
			else if( NbrMatchsFound > 1 ) {
				g_print( "found %d matches in CDDB. Using first one.\n", NbrMatchsFound );
				// OK -> return( 0 );
			}
			
			if( cddb_read( p_cddb, p_disc ) < 0 ) {
				g_print( "LA LIAISON INTERNET EST ABSENTE\n" );
 				strcpy( VarCddb . Message, _("La liaison internet est absente") );
				// cddb_disc_destroy( p_disc );
				// cddb_destroy( p_cddb );
				RetValue = 4;
				// return( RetValue );
			}

			g_print( "\n-------------------------------------------------\n" );
			if( 4 != RetValue && 5 != RetValue )
				VarCddb . TitleCD = g_strdup( cddb_disc_get_title( p_disc ));
			else	VarCddb . TitleCD = g_strdup( "TitleCD" );
			g_print( "TITLE  = %s\n", VarCddb . TitleCD);
			
			if( 4 != RetValue && 5 != RetValue )
				VarCddb . Artiste = g_strdup( cddb_disc_get_artist( p_disc ));
			else	VarCddb . Artiste = g_strdup( " " );
			g_print( "ARTIST = %s\n", VarCddb . Artiste);
			
			if( 4 != RetValue && 5 != RetValue )
				VarCddb . StrGenre = g_strdup( cddb_disc_get_genre( p_disc ));
			else	VarCddb . StrGenre = g_strdup( "StrGenre" );
			g_print( "GENRE  = %s\n", VarCddb . StrGenre);
			
			if( 4 != RetValue && 5 != RetValue )
				VarCddb . NumYear = cddb_disc_get_year( p_disc );
			else	VarCddb . NumYear = 1962;
			VarCddb . StrYear = g_strdup_printf( "%d", VarCddb . NumYear );
			g_print( "YEAR   = %d\n", VarCddb . NumYear);
			
			// g_sprintf (CdToc->Str_ID_CD, "%08lx", CdToc->Num_ID_CD);
			if( 4 != RetValue && 5 != RetValue )
				g_print( "DISCID = %x\n", cddb_disc_get_discid( p_disc ));
			else	g_print( "DISCID = 0\n");
			
			len = VarCddb . VarStructCddb [ TotalTracks ] . SectorCdParanoia - VarCddb . VarStructCddb [ 0 ] . SectorCdParanoia;
			VarCddb . StrDureeCd = g_strdup_printf( "%02d:%02d", ( len / 60 ) / 75,  ( len / 75 ) % 60 );
			g_print( "DUREE  = %s\n", VarCddb . StrDureeCd );
			g_print( "-------------------------------------------------\n" );
			len = 0;
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				
				g_print( "%02d  ", Track +1 );
								
				// CONNECT IS OK
				if( 4 != RetValue && 5 != RetValue ) {
					
					cddb_track_t *t = cddb_disc_get_track( p_disc, Track );
				
					if( NULL == t ) continue;
				
					// ARTISTE
					VarCddb . VarStructCddb [ Track ] . Artist = g_strdup( cddb_track_get_artist( t ));
					g_print( "%s\t", VarCddb . VarStructCddb [ Track ] . Artist);
				
					// TITRE PISTE
					VarCddb . VarStructCddb [ Track ] . Title = g_strdup( cddb_track_get_title( t ));
					g_print( "%s       ", VarCddb . VarStructCddb [ Track ] . Title);
					
				}
				// NO CONNECT
				else {
					// ARTISTE
					VarCddb . VarStructCddb [ Track ] . Artist = g_strdup( "Artist" );
					g_print( "%s\t", VarCddb . VarStructCddb [ Track ] . Artist);
				
					// TITRE PISTE
					VarCddb . VarStructCddb [ Track ] . Title = g_strdup( "Title" );
					g_print( "%s", VarCddb . VarStructCddb [ Track ] . Title);
				}
				
				g_print( "\n" );
			}
			
			g_print( "-------------------------------------------------\n\n" );
			g_print( "cddb_disc_get_category_str() = %s\n", cddb_disc_get_category_str( p_disc ));
			g_print( "cddb_disc_get_category()     = %d\n", cddb_disc_get_category( p_disc ));
			// g_print( "DATAS                        = %s\n", cddb_disc_get_ext_data( p_disc ));
			
			if( 4 != RetValue && 5 != RetValue )
				VarCddb . StrGenre = g_strdup( cddb_disc_get_genre( p_disc ));
			else	VarCddb . StrGenre = g_strdup( "Rock" );
			g_print( "VarCddb . StrGenre           = %s\n", VarCddb . StrGenre);
			VarCddb . NumGenre = tags_get_genre_by_value( VarCddb . StrGenre );
			g_print( "VarCddb . NumGenre           = %d\n", VarCddb . NumGenre);
			
			// cddb_disc_print (p_disc);
			
			// BASE ADJUSTMENT FROM CDPARANOIA
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				VarCddb . VarStructCddb [ Track ] . Sector = VarCddb . VarStructCddb [ Track ] . SectorCdParanoia;
			}
			VarCddb . TotalTracks = TotalTracks;
			g_print( "\nRECTIF FOR : DataTrack\n" );
			g_print( "VarCddb . TotalTracks       = %d\n",  VarCddb . TotalTracks );
			g_print( "TotalTracks with cdparanoia = %d\n\n",  TotalTracks );
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				 // DUREE D'ECOUTE DES PISTES
				min = VarCddb . VarStructCddb [ Track +1 ] . Sector - VarCddb . VarStructCddb [ Track ] . Sector;
				VarCddb . VarStructCddb [ Track ] . Duree = g_strdup_printf( "%02d:%02d", ( min / 60 ) / 75, ( min / 75 ) % 60 );
			}
			
			// SET DATAS FOR CD_CUE
			cdcue_alloc_base_ioctl( VarCddb . TotalTracks );
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				BaseIoctl.Datas[ Track ].length = -1;
				BaseIoctl.Datas[ Track ].begin  = -1;
			}

			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
		
				BaseIoctl.Datas[ Track ].length = VarCddb . VarStructCddb [ Track +1 ] . Sector - VarCddb . VarStructCddb [ Track ] . Sector;
				TempDiv = BaseIoctl.Datas[ Track ].length / 75;
				TempMod = BaseIoctl.Datas[ Track ].length % 75;
				BaseIoctl.Datas[ Track ].length_min  = TempDiv / 60;
				BaseIoctl.Datas[ Track ].length_sec  = TempDiv % 60;
				BaseIoctl.Datas[ Track ].length_cent = TempMod;
			}
			BaseIoctl.Datas[ 0 ].begin = VarCddb . VarStructCddb [ 0 ] . Sector % 75;
	
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
		
				BaseIoctl.Datas[ Track +1 ].begin = BaseIoctl.Datas[ Track ].length + BaseIoctl.Datas[ Track ].begin;
				TempDiv = BaseIoctl.Datas[ Track ].begin / 75;
				TempMod = BaseIoctl.Datas[ Track ].begin % 75;
				BaseIoctl.Datas[ Track ].begin_min  = TempDiv / 60;
				BaseIoctl.Datas[ Track ].begin_sec  = TempDiv % 60;
				BaseIoctl.Datas[ Track ].begin_cent = TempMod;
			}
			BaseIoctl.Datas[ VarCddb . TotalTracks ].begin  = BaseIoctl.Datas[ Track ].length + BaseIoctl.Datas[ Track ].begin;
			BaseIoctl.Datas[ VarCddb . TotalTracks ].begin -= BaseIoctl.Datas[ 0 ].begin;
			TempDiv = BaseIoctl.Datas[ VarCddb . TotalTracks ].begin / 75;
			TempMod = BaseIoctl.Datas[ VarCddb . TotalTracks ].begin % 75;
			BaseIoctl.Datas[ VarCddb . TotalTracks ].begin_min  = TempDiv / 60;
			BaseIoctl.Datas[ VarCddb . TotalTracks ].begin_sec  = TempDiv % 60;
			BaseIoctl.Datas[ VarCddb . TotalTracks ].begin_cent = TempMod;
			
			cdcue_make_cue();

			BaseIoctl.Performer = g_strdup( VarCddb . TitleCD );
			BaseIoctl.Title     = g_strdup( VarCddb . TitleCD );
			BaseIoctl.File      = g_strdup_printf ("%s.wav", VarCddb . TitleCD);
	
			for( Track = 0; Track < BaseIoctl.TotalTracks; Track++ ) {
				BaseIoctl.Cue [ Track ].Performer = g_strdup( VarCddb . TitleCD );
				BaseIoctl.Cue [ Track ].Title     = g_strdup( VarCddb . VarStructCddb [ Track ] . Title );
			}


			cdcue_print_base_ioctl();
			
			cddb_disc_destroy( p_disc );
		}
		cddb_destroy( p_cddb );
	}
	
	return( RetValue );
}
// P_ETAT = 
// 	->0  OK
// 	  1  PROBLEME D'ALLOCATION MEMOIRE
// 	  2  CD NON AUDIO OU CD ABSENT DU LECTEUR
// 	  3  LE CD DANS LECTEUR N'EST PAS UN CD-AUDIO
//	->4  PAS DE CONNEXION WEB
//	->5  PAS DE BASE POUR CE DISQUE
//	  6  ERREUR DE LECTURE DEPUIS LE CD
//	  7  CD ABSENT DU LECTEUR
// 
void CdCddb_set_toc_to_glist_audio (gint p_etat)
{
	gint	NumList;
	AUDIO	*Audio = NULL;
	
	// REMOVE DATAS entete
	CdCddb_init_entete_cd();
	
	EnteteCD.Message = g_strdup( VarCddb . Message );
	
	// 
	switch( p_etat ) {
	case 0 :	// OK
		EnteteCD.TitleCD     = g_strdup_printf( "%s  -  %s", VarCddb . Artiste, VarCddb . TitleCD );
		EnteteCD.Artiste     = g_strdup( VarCddb . TitleCD );
		EnteteCD.TotalTracks = VarCddb . TotalTracks;
		EnteteCD.NumGenre    = VarCddb . NumGenre;
		EnteteCD.StrGenre    = g_strdup( VarCddb . StrGenre );
		EnteteCD.NumYear     = VarCddb . NumYear;
		EnteteCD.StrYear     = g_strdup( VarCddb . StrYear );
		EnteteCD.StrDureeCd    = g_strdup( VarCddb . StrDureeCd );
		break;
	case 4 :	// PAS DE CONNEXION WEB
		EnteteCD.TitleCD     = g_strdup( "Track" );
		EnteteCD.Title       = g_strdup( "Track" );
		EnteteCD.Artiste     = g_strdup( "Track" );
		EnteteCD.TotalTracks = VarCddb . TotalTracks;
		EnteteCD.StrDureeCd    = g_strdup( VarCddb . StrDureeCd );
		break;
	case 5 :	// PAS DE BASE POUR CE DISQUE
		EnteteCD.TitleCD     = g_strdup( "Track" );
		EnteteCD.Title       = g_strdup( "Track" );
		EnteteCD.Artiste     = g_strdup( "Track" );
		EnteteCD.TotalTracks = VarCddb . TotalTracks;
		EnteteCD.StrDureeCd    = g_strdup( VarCddb . StrDureeCd );
		break;
	}
	
	for( NumList = 0; NumList < VarCddb . TotalTracks; NumList++ ) {

		Audio = ( AUDIO * )g_malloc0( sizeof( AUDIO ));
		
		Audio->EtatPlay                 = CD_ETAT_PLAY_ATTENTE;

		Audio->Bool_Delette_Flac	= FALSE;
		Audio->Bool_Delette_Wav		= FALSE;
		Audio->Bool_Delette_Mp3		= FALSE;
		Audio->Bool_Delette_Ogg		= FALSE;
		Audio->Bool_Delette_M4a		= FALSE;
		Audio->Bool_Delette_Mpc		= FALSE;
		Audio->Bool_Delette_Ape		= FALSE;
		Audio->Bool_Delette_WavPack	= FALSE;

		Audio->EtatSelection_Flac	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_Wav	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_Mp3	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_Ogg	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_M4a	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_Mpc	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_Ape	= CD_ETAT_ATTENTE;
		Audio->EtatSelection_WavPack	= CD_ETAT_ATTENTE;

		Audio->Etat_Flac		= NONE;
		Audio->Etat_Wav			= NONE;
		Audio->Etat_Mp3			= NONE;
		Audio->Etat_Ogg			= NONE;
		Audio->Etat_m4a			= NONE;
		Audio->Etat_Mpc			= NONE;
		Audio->Etat_Ape			= NONE;
		Audio->Etat_WavPack		= NONE;
		Audio->EtatNormalise		= CD_NORM_PEAK_NONE;

		Audio->PathName_Dest_Flac	= NULL;
		Audio->PathName_Dest_Wav	= NULL;
		Audio->PathName_Dest_Mp3	= NULL;
		Audio->PathName_Dest_Ogg	= NULL;
		Audio->PathName_Dest_M4a	= NULL;
		Audio->PathName_Dest_Mpc	= NULL;
		Audio->PathName_Dest_Ape	= NULL;
		Audio->PathName_Dest_WavPack	= NULL;
				
		Audio->tags = ( TAGS * )tags_alloc( TRUE );
	
		Audio->Num_Track		= NumList + 1;
		Audio->Str_Track		= g_strdup_printf ("%02d", NumList + 1);
		Audio->Duree			= g_strdup( VarCddb . VarStructCddb [ NumList ] . Duree );
		Audio->NameSong			= g_strdup_printf ("Track_%02d", NumList + 1);
		
		// OK
		if( 0 == p_etat ) {
			
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Title ) {
				g_free( Audio->NameSong );	Audio->NameSong = NULL;
				Audio->NameSong = g_strdup( VarCddb . VarStructCddb [ NumList ] . Title );
			}
			Audio->tags = (TAGS *)tags_remove (Audio->tags);
			Audio->tags = (TAGS *)tags_alloc (TRUE);
			Audio->tags->Album = g_strdup( VarCddb . TitleCD );
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Title ) {
				Audio->tags->Artist = g_strdup( VarCddb . VarStructCddb [ NumList ] . Artist );
				Audio->tags->Title  = g_strdup( VarCddb . VarStructCddb [ NumList ] . Title );
			}
			Audio->tags->Year      = g_strdup( EnteteCD.StrYear );
			Audio->tags->IntYear   = EnteteCD.NumYear;
			Audio->tags->IntGenre  = EnteteCD.NumGenre;
			Audio->tags->Genre     = g_strdup( EnteteCD.StrGenre );
			Audio->tags->Number    = g_strdup_printf ("%d", NumList +1);
			Audio->tags->IntNumber = NumList +1;
			Audio->tags->Comment   = g_strdup ("By Xcfa");
		}
		
		// PAS DE CONNEXION WEB
		else if( 4 == p_etat ) {
			
			Audio->tags = (TAGS *)tags_remove (Audio->tags);
			Audio->tags = (TAGS *)tags_alloc (TRUE);
			
			Audio->tags->Album       = g_strdup ("Track");
			Audio->tags->Artist      = g_strdup ("");
			Audio->tags->Title       = g_strdup_printf ("Track_%02d", NumList +1);
			Audio->tags->Number      = g_strdup_printf ("%d", NumList +1);
			Audio->tags->IntNumber   = NumList +1;
			Audio->tags->Genre       = g_strdup ("A Cappella");
			Audio->tags->IntGenre    = 123;
			Audio->tags->Year        = g_strdup ("1962");
			Audio->tags->IntYear     = 1962;
			Audio->tags->Comment     = g_strdup ("By Xcfa");
			Audio->tags->Description = g_strdup ("By Xcfa");
		}
		
		// PAS DE BASE POUR CE DISQUE
		else if( 5 == p_etat ) {
				
			Audio->tags = (TAGS *)tags_remove (Audio->tags);
			Audio->tags = (TAGS *)tags_alloc (TRUE);
			
			Audio->tags->Album       = g_strdup ("Track");
			Audio->tags->Artist      = g_strdup ("");
			Audio->tags->Title       = g_strdup_printf ("Track_%02d", NumList +1);
			Audio->tags->Number      = g_strdup_printf ("%d", NumList +1);
			Audio->tags->IntNumber   = NumList +1;
			Audio->tags->Genre       = g_strdup ("A Cappella");
			Audio->tags->IntGenre    = 123;
			Audio->tags->Year        = g_strdup ("1962");
			Audio->tags->IntYear     = 1962;
			Audio->tags->Comment     = g_strdup ("By Xcfa");
			Audio->tags->Description = g_strdup ("By Xcfa");
		}
		
		EnteteCD.GList_Audio_cd = g_list_append (EnteteCD.GList_Audio_cd, (AUDIO *)Audio);

	}
}
// 
// 
gboolean CdCddb_read_toc (void)
{
	gint	Etat;
	
	// 
	Etat = CdCddb_read_datas();
	
	// 
	CdCddb_set_toc_to_glist_audio( Etat );
	
	// REMOVE VarCddb
	CdCddb_remove_struct();
	
	if( 0 == Etat || 4 == Etat || 5 == Etat ) {
		return( TRUE );
	}
	return ( FALSE );
}



gboolean bool_cancel_cddb = FALSE;	/* FALSE = continue ELSE STOP */

// int strcasecmp (const char *s1, const char *s2);
// int strncasecmp (const char *s1, const char *s2, size_t n);




// Si 'FLAG == FALSE', force l'arret de la recherche sur le serveur freedb
//
void CdCddb_set_bool_cancel_cddb (gboolean FLAG)
{
	/*PRINT_FUNC_LF();*/
	bool_cancel_cddb = FLAG;
}
// 
// 
gboolean CdCddb_get_bool_cancel_cddb (void)
{
	return (bool_cancel_cddb);
}
//
//
gboolean CdCddb_test_access_web (void)
{
	// PRINT_FUNC_LF();
	
	if (libutils_find_file ("wget") == FALSE) {
		g_print ("WGET absent !\n");
		return (FALSE);
	}
	return (GetInfo_wget ());
}





















